import { Component, EventEmitter, Input, Output } from '@angular/core';
import { EmployeesForAQualificationDTO, QualificationGetDTO } from '../../models/skill';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { SkillService } from '../../service/skill.service';
import { EmployeeService } from '../../service/employee.service';
import { concatMap, every, from, lastValueFrom, Observable, of, switchMap, take, tap } from 'rxjs';
import { EmployeeNameDataDTO, EmployeeResponseDTO } from '../../models/mitarbeiter';
import { AsyncPipe } from '@angular/common';
import { Router } from '@angular/router';

@Component({
  selector: 'app-qualifikation-form',
  standalone: true,
  imports: [ReactiveFormsModule, AsyncPipe],
  templateUrl: './qualifikation-form.component.html',
  styleUrl: './qualifikation-form.component.css'
})
export class QualifikationFormComponent {
  @Input() skill!: QualificationGetDTO;
  @Output() skillChange = new EventEmitter<QualificationGetDTO>();

  public skillForm!: FormGroup;
  public addableEmployees: Array<EmployeeResponseDTO> = [];
  public addedEmployees: Array<EmployeeNameDataDTO> = [];
  errorMessages: Record<string, string> = {};


  constructor(private skillService: SkillService, private employeeService: EmployeeService, private router: Router) { }

  private validationErrorMessages: Record<string, string> = {
    required: "This field is required",
  };

  updateErrorMessages(): void {
    this.errorMessages = {};

    Object.keys(this.skillForm.controls).forEach(field => {
      const control = this.skillForm.get(field);

      if (control && control.errors) {
        this.errorMessages[field] = Object.keys(control.errors)
          .map(errorKey => this.validationErrorMessages[errorKey] || `Unknown error: ${errorKey}`)
          .join(' ');
      }
    });
  }

  setUpForm() {
    this.skillForm = new FormGroup({
      name: new FormControl(this.skill.skill, Validators.required),
      newEmployee: new FormControl(),
    });
  }


  addEmployee() {
    const employeeId = Number(this.skillForm.get("newEmployee")?.value);
    const employee = this.addableEmployees.find(emp => emp.id === employeeId);

    if (employee) {
      this.addableEmployees = this.addableEmployees.filter(emp => emp.id !== employeeId);

      this.addedEmployees.push(employee);
    }
  }

  removeEmployee(employeeId: number) {
    const employee = this.addedEmployees.find(emp => emp.id === employeeId);

    if (employee) {
      this.addedEmployees = this.addedEmployees.filter(emp => emp.id !== employeeId);

      this.employeeService.getEmployeeById(employee.id).subscribe(employeeDto => {
        this.addableEmployees.push(employeeDto);
      });
    }
  }

  returnToSkillList() {
    this.router.navigate(["qualifikationsverwaltung"]);
  }

  updateEmployeeLists() {
    if (this.skill.id != -1) {
      this.employeeService.getAllEmployees().subscribe(employees => {
        this.addableEmployees = employees;

        this.skillService.getEmployeesBySkill(this.skill.id).subscribe(addedEmployeesResponse => {
          this.addedEmployees = addedEmployeesResponse.employees;

          this.addableEmployees = this.addableEmployees.filter(employee => {
            return !this.addedEmployees.some(added => added.id === employee.id);
          });
        });
      });
    }
  }

  submit() {
    this.updateErrorMessages();
    if (!this.skillForm.valid) {
      return;
    }

    for (const employee of this.addedEmployees) {
      this.employeeService.getEmployeeById(employee.id).subscribe(employeeResponse => {
        this.employeeService.addSkillToEmployee(this.skill.id, employeeResponse);
      });
    }

    for (const employee of this.addableEmployees) {
      this.employeeService.removeSkillFromEmployee(this.skill.id, employee.id);
    }

    this.skill.skill = this.skillForm.get("name")?.value;
    this.skillChange.emit(this.skill);

    this.router.navigate(["/qualifikationen"]);
  }

  ngOnChanges(): void {
    this.setUpForm();
    this.updateEmployeeLists();
  }

  ngOnInit() {
    this.setUpForm();
    this.updateEmployeeLists();
  }
}
